package Bread::Board::LifeCycle::TouchFile;
{
  $Bread::Board::LifeCycle::TouchFile::VERSION = '0.132280';
}
# ABSTRACT: lifecycle by file mtime
use Moose::Role;
with 'Bread::Board::LifeCycle::Singleton';

has timestamp => (
    traits    => ['NoClone'],
    is        => 'rw',
    isa       => 'Int',
    predicate => 'has_timestamp',
    clearer   => 'flush_timestamp'
);

# これにしたいが、LifeCycleはServiceのattrのtriggerでapplyされ
# BUILDに間に合わないので、dependenciesから取ることにする。
#has touch_file => (
#    traits    => ['NoClone'],
#    is        => 'ro',
#    isa       => 'Str',
#    required  => 1,
#);

around 'get' => sub {
    my $orig = shift;
    my $self = shift;

    my $touch_file = $self->params->{touch_file} or die "touch_file parameter required.";

    -f $touch_file or do { open(my $fh, '>', $touch_file) } or die "touch_file $touch_file is not writable file.";

    my $mtime = (stat($touch_file))[9];

    if (!$self->timestamp or $self->timestamp < $mtime) {
        $self->flush_instance;
        $self->timestamp($mtime);
    }

    $self->$orig(@_);
};

no Moose::Role;
1;

__END__

=pod

=head1 NAME

Bread::Board::LifeCycle::TouchFile - lifecycle by file mtime

=head1 VERSION

version 0.132280

=head1 SYNOPSIS

    use Bread::Board;

    my $c = container 'ResettableRandomNumber' => as {
        service touch_file => 'var/random_number_touch_file';
        service random_number => (
            lifecycle => 'TouchFile',
            dependencies => {
                touch_file => depends_on('touch_file'),
            },
            block => sub {
                rand;
            },
        );
    };

    say $c->resolve( service => 'ResettableRandomNumber' ); # 0.952260206443931
    say $c->resolve( service => 'ResettableRandomNumber' ); # 0.952260206443931

    touch 0, time+1, 'var/random_number_touch_file';

    say $c->resolve( service => 'ResettableRandomNumber' ); # 0.796248696946889 new number!

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
